/*	$Id: status.c,v 1.41 2002/05/21 01:32:40 riq Exp $	*/
/* Tenes Empanadas Graciela
 *
 * Copyright (C) 2000 Ricardo Quesada
 *
 * Author: Ricardo Calixto Quesada <rquesada@core-sdi.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; only version 2 of the License
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */
/**
 * @file status.c
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <assert.h>
#include <gnome.h>

#include "gui.h"
#include "client.h"
#include "support.h"
#include "interface.h"
#include "callbacks.h"
#include "status.h"
#include "colors.h"
#include "priv.h"
#include "fonts.h"

extern TTheme gui_theme;



/* some defines for the position */
#define RECT_SIZE (8)
#define PLAYERS_COLORS_OFFSET (246)
#define ROUND_OFFSET (410)
#define OFFSET_LEFT (6)
#define OFFSET_RIGHT (-8)

GtkWidget *status_dialog=NULL;
static GtkWidget *ministatus = NULL;
static int linesindisplay=0;

static GtkWidget *status_clist=NULL;
static GtkWidget* mainstatus_canvas = NULL;

static GnomeCanvasItem *players_color[TEG_MAX_PLAYERS];
static GnomeCanvasItem *color_started_item;
static GnomeCanvasItem *round_number_item;

static gint status_boton_clicked_cb(GtkWidget *area, GdkEventExpose *event, gpointer user_data)
{
	out_status();
	return FALSE;
}

static TEG_STATUS status_paint_color( int color, GdkPixmap **pixmap )
{
	int i, h, w;

	assert( pixmap );

	i = (color<0 || color>=TEG_MAX_PLAYERS) ? TEG_MAX_PLAYERS : color;

	*pixmap = gdk_pixmap_new(status_dialog->window,
		48, 16, gtk_widget_get_visual(status_dialog)->depth);

	if( *pixmap == NULL )
		return TEG_STATUS_ERROR;


	gdk_gc_set_foreground(g_colors_gc, colors_get_player_from_color(color));
	gdk_draw_rectangle( *pixmap, g_colors_gc, TRUE, 0, 0, 47, 15);

	gdk_gc_set_foreground(g_colors_gc, colors_get_common(COLORS_BLACK));
	gdk_draw_rectangle( *pixmap, g_colors_gc, FALSE, 0, 0, 47, 15);

	gdk_gc_set_foreground(g_colors_gc, colors_get_player_ink_from_color(color));



	h = gdk_string_height (g_pixmap_font10, _(g_colores[i]) );
	w = gdk_string_width  (g_pixmap_font10, _(g_colores[i]) );

	gdk_draw_string( *pixmap, g_pixmap_font10, g_colors_gc, 
			((48 - w )/2),
			((16 - h)/2) + h, _(g_colores[i]));
	return TEG_STATUS_SUCCESS;
}

TEG_STATUS status_turn_color(PJUGADOR pJ, GdkPixmap **pixmap)
{
	int i;

	assert( pixmap );
	assert( pJ );


	if( pJ->empezo_turno )
		i = pJ->numjug;
	else
		i = -1;

	*pixmap = gdk_pixmap_new(status_dialog->window,
		16, 16, gtk_widget_get_visual(status_dialog)->depth);

	if( *pixmap == NULL )
		return TEG_STATUS_ERROR;

	gdk_gc_set_foreground(g_colors_gc, colors_get_player(i));
	gdk_draw_rectangle( *pixmap, g_colors_gc, TRUE, 0, 0, 15, 15);

	gdk_gc_set_foreground(g_colors_gc, colors_get_common(COLORS_BLACK));
	gdk_draw_rectangle( *pixmap, g_colors_gc, FALSE, 0, 0, 15, 15);

	return TEG_STATUS_SUCCESS;
}


/* view the status of players */
void status_view()
{
	gint i ;
	char *titles[S_CLIST_N_LAST] = {
		_("Color"), _("Number"),_("Name"),_("Score"), _("Address"), _("Human"), _("Countries"), _("Armies"), _("Cards"),_("Status"), _("Who started"),
	};

	if( status_dialog == NULL) {

		status_dialog = teg_dialog_new(_("Status of Players"),_("Status of Players")); 

		gtk_window_set_transient_for (GTK_WINDOW(status_dialog), NULL);

		gnome_dialog_append_buttons(GNOME_DIALOG(status_dialog),
				GNOME_STOCK_PIXMAP_REFRESH,
				GNOME_STOCK_BUTTON_CLOSE,
				NULL );

		gnome_dialog_close_hides( GNOME_DIALOG(status_dialog), TRUE );
		gnome_dialog_set_default(GNOME_DIALOG(status_dialog),1);

		/* signals de los botones */
		gnome_dialog_button_connect (GNOME_DIALOG(status_dialog),
						0, GTK_SIGNAL_FUNC(status_boton_clicked_cb),status_dialog);
		gnome_dialog_button_connect (GNOME_DIALOG(status_dialog),
						1, GTK_SIGNAL_FUNC(dialog_close),status_dialog);
		gnome_dialog_set_default( GNOME_DIALOG(status_dialog),1);


		status_clist = gtk_clist_new_with_titles (S_CLIST_N_LAST, titles);
		
		gtk_clist_set_row_height (GTK_CLIST (status_clist), 18);
//		gtk_widget_set_usize (status_clist, -1, 300);

		gtk_box_pack_start_defaults( GTK_BOX(GNOME_DIALOG(status_dialog)->vbox), GTK_WIDGET(status_clist));

		gtk_clist_set_selection_mode (GTK_CLIST (status_clist), GTK_SELECTION_EXTENDED);
		for (i = 0; i < S_CLIST_N_LAST; i++) {
			gtk_clist_set_column_justification (GTK_CLIST (status_clist), i, GTK_JUSTIFY_CENTER);
			gtk_clist_set_column_auto_resize (GTK_CLIST (status_clist), i, TRUE);
		}

		linesindisplay = 0;
	}
	status_fill_clist( status_clist);
	gtk_widget_show_all(status_dialog);
	raise_and_focus(status_dialog);
	out_status();
}


TEG_STATUS status_fill_clist( GtkWidget *clist )
{
#define S_CLIST_MAX (100)
	PJUGADOR j;
	PLIST_ENTRY l = g_list_jugador.Flink;
	char clist_text[S_CLIST_N_LAST][S_CLIST_MAX];
	char *clist_texts[S_CLIST_N_LAST];
	int i, row;
	GdkPixmap *pixmap;
	
	gtk_clist_freeze(GTK_CLIST(clist));

	for(i=0;i<S_CLIST_N_LAST;i++) {
		int b;

		clist_texts[i] = clist_text[i];
		memset( clist_text[i],0,S_CLIST_MAX);

		b = gui_private.status_show & (1 <<i ) ? 1 : 0;
		gtk_clist_set_column_visibility( GTK_CLIST(clist), i, b );
	}

	row = 0;
	while( !IsListEmpty( &g_list_jugador ) && (l != &g_list_jugador) ) {
		j = (PJUGADOR) l;

		/* sprintf(clist_texts[0],"%s",g_colores[j->color]); */
		snprintf(clist_texts[S_CLIST_N_NUMBER],S_CLIST_MAX-1,"%d",j->numjug);
		snprintf(clist_texts[S_CLIST_N_NAME],S_CLIST_MAX-1,"%s",j->nombre);
		snprintf(clist_texts[S_CLIST_N_SCORE],S_CLIST_MAX-1,"%d",j->score);
		snprintf(clist_texts[S_CLIST_N_ADDR],S_CLIST_MAX-1,"%s",j->addr);
		snprintf(clist_texts[S_CLIST_N_HUMAN],S_CLIST_MAX-1,"%s",(j->human?_("yes"):_("no")) );
		snprintf(clist_texts[S_CLIST_N_COUNTRIES],S_CLIST_MAX-1,"%d",j->tot_paises);
		snprintf(clist_texts[S_CLIST_N_ARMIES],S_CLIST_MAX-1,"%d",j->tot_ejercitos);
		snprintf(clist_texts[S_CLIST_N_CARDS],S_CLIST_MAX-1,"%d",j->tot_tarjetas);
		snprintf(clist_texts[S_CLIST_N_STATUS],S_CLIST_MAX-1,"%s",_(g_estados[j->estado]));

		if (linesindisplay < row+1) {
			gtk_clist_append (GTK_CLIST (clist), clist_texts);
			linesindisplay++;
		}
		else {
	 		for(i=1;i<S_CLIST_N_LAST;i++)
	   			gtk_clist_set_text(GTK_CLIST (clist), row, i, clist_texts[i]);
		}

		if( status_paint_color( j->color, &pixmap ) == TEG_STATUS_SUCCESS )
			gtk_clist_set_pixmap(GTK_CLIST(clist), row, S_CLIST_N_COLOR, pixmap, NULL);

		if( status_turn_color( j, &pixmap ) == TEG_STATUS_SUCCESS )
			gtk_clist_set_pixmap(GTK_CLIST(clist), row, S_CLIST_N_WHO, pixmap, NULL);

		l = LIST_NEXT(l);
		row ++;
	}
	while (linesindisplay > row)
		gtk_clist_remove(GTK_CLIST(clist), --linesindisplay);

	gtk_clist_thaw(GTK_CLIST(clist));

	return TEG_STATUS_SUCCESS;
}

TEG_STATUS status_update()
{
	if( status_dialog != NULL)
		status_fill_clist( status_clist );

	return TEG_STATUS_SUCCESS;
}

TEG_STATUS ministatus_update()
{
	if( ministatus == NULL )
		return TEG_STATUS_ERROR;

	gtk_widget_draw( ministatus, NULL);

	return TEG_STATUS_SUCCESS;
}

static gint ministatus_expose_cb(GtkWidget *area, GdkEventExpose *event, gpointer user_data)
{
	static GdkGC *ms_gc = NULL;
	int i=0;

	if( area == NULL )
		return FALSE;

	if( area->window == NULL )
		return FALSE;;

	if( ms_gc == NULL )
		ms_gc = gdk_gc_new(area->window);

	if( ESTADO_GET() == JUG_ESTADO_DESCONECTADO || g_juego.observer )
		i = -1;
	else
		i = g_juego.numjug;

	gdk_gc_set_foreground(ms_gc, colors_get_player(i));
	gdk_draw_arc( area->window, ms_gc, TRUE, 0, 3, 10, 10, 0, 360 * 64);

	gdk_gc_set_foreground(ms_gc, colors_get_common(COLORS_BLACK));
	gdk_draw_arc( area->window, ms_gc, FALSE, 0, 3, 10, 10, 0, 360* 64);

	return FALSE;
}

GtkWidget *ministatus_build()
{
	if( ministatus == NULL ) {
		ministatus = gtk_drawing_area_new();

		gtk_signal_connect(GTK_OBJECT(ministatus), "expose_event",
			   GTK_SIGNAL_FUNC(ministatus_expose_cb), NULL);
	}
	gtk_widget_set_usize(ministatus, 15, -1);
	gtk_widget_show( ministatus );

	return ministatus;
}


/*
 * Main Status
 */

#define MAINSTATUS_X (642)
#define MAINSTATUS_Y (30)
TEG_STATUS mainstatus_create( GtkWidget **window )
{
	int i;
	int failed=1;

	if( mainstatus_canvas )
		goto error;

	mainstatus_canvas = gnome_canvas_new();
	if( ! mainstatus_canvas )
		goto error;

	/* load colors for started_item, and player_colors */
	colors_load_images();

	gtk_widget_set_usize ( mainstatus_canvas, MAINSTATUS_X, MAINSTATUS_Y + 1 );
	gnome_canvas_set_scroll_region (GNOME_CANVAS (mainstatus_canvas), 0, 0, MAINSTATUS_X, MAINSTATUS_Y);

	/* background */
	if( gui_theme.toolbar_custom && gui_theme.toolbar_name )
	{
		GdkImlibImage *im;
		char *filename = theme_load_file( gui_theme.toolbar_name );
		im = gdk_imlib_load_image (filename);

		if( im ) {
			gnome_canvas_item_new(
				gnome_canvas_root(GNOME_CANVAS(mainstatus_canvas)),
				gnome_canvas_image_get_type (),
				"image", im,
				"x", 0.0,
				"y", 0.0,
/*				"width", (double) im->rgb_width, */
/*				"height", (double) im->rgb_height, */
				"width", (double) MAINSTATUS_X,
				"height", (double) MAINSTATUS_Y,
				"anchor", GTK_ANCHOR_NW,
				NULL);

			failed = 0;
		}
	}

	/* load default background */
	if( failed ) {
		gnome_canvas_item_new(
			gnome_canvas_root(GNOME_CANVAS(mainstatus_canvas)),
			gnome_canvas_rect_get_type (),
			"x1", 0.0,
			"y1", 0.0,
			"x2", (double) MAINSTATUS_X,
			"y2", (double) MAINSTATUS_Y,
			"fill_color","light green",
			"outline_color","black",
			NULL);
	}

	/* round started by */
	gnome_canvas_item_new(
		gnome_canvas_root(GNOME_CANVAS(mainstatus_canvas)),
		gnome_canvas_text_get_type(),
		"text",_("Round started by:"),
		"x", (double) ROUND_OFFSET,
		"y", (double) 3,
		"x_offset", (double) -1,
		"y_offset", (double) -1,
		"font", HELVETICA_10_FONT,
		"fill_color", gui_theme.toolbar_custom && gui_theme.toolbar_text_color ? gui_theme.toolbar_text_color : "black",
		"anchor",GTK_ANCHOR_NE,
		NULL);

	color_started_item = gnome_canvas_item_new(
		gnome_canvas_root(GNOME_CANVAS(mainstatus_canvas)),
		gnome_canvas_image_get_type(),
		"image", g_color_circles[TEG_MAX_PLAYERS],
		"x", (double) ROUND_OFFSET + 4,
		"y", (double) 4,
		"width", (double) g_color_circles[TEG_MAX_PLAYERS]->rgb_width,
		"height", (double) g_color_circles[TEG_MAX_PLAYERS]->rgb_height,
		"anchor",GTK_ANCHOR_NW,
		NULL);
	gnome_canvas_item_hide( color_started_item );

	/* round number */
	gnome_canvas_item_new(
		gnome_canvas_root(GNOME_CANVAS(mainstatus_canvas)),
		gnome_canvas_text_get_type(),
		"text",_("Round number:"),
		"x", (double) ROUND_OFFSET,
		"y", (double) MAINSTATUS_Y/2 + 1,
		"x_offset", (double) -1,
		"y_offset", (double) -1,
		"font", HELVETICA_10_FONT,
		"fill_color", gui_theme.toolbar_custom && gui_theme.toolbar_text_color ? gui_theme.toolbar_text_color : "black",
		"anchor",GTK_ANCHOR_NE,
		NULL);

	round_number_item = gnome_canvas_item_new(
		gnome_canvas_root(GNOME_CANVAS(mainstatus_canvas)),
		gnome_canvas_text_get_type(),
		"text",_("?"),
		"x", (double) ROUND_OFFSET + 4,
		"y", (double) MAINSTATUS_Y/2 + 2,
		"x_offset", (double) -1,
		"y_offset", (double) -1,
		"font", HELVETICA_12_FONT,
		"fill_color", gui_theme.toolbar_custom && gui_theme.toolbar_text_color ? gui_theme.toolbar_text_color : "black",
		"anchor",GTK_ANCHOR_NW,
		NULL);

	/* players turn */
	gnome_canvas_item_new(
		gnome_canvas_root(GNOME_CANVAS(mainstatus_canvas)),
		gnome_canvas_text_get_type(),
		"text",_("Players turn:"),
		"x", (double) PLAYERS_COLORS_OFFSET - 4,
		"y", (double) 3,
		"x_offset", (double) -1,
		"y_offset", (double) -1,
		"font", HELVETICA_10_FONT,
		"fill_color", gui_theme.toolbar_custom && gui_theme.toolbar_text_color ? gui_theme.toolbar_text_color : "black",
		"anchor",GTK_ANCHOR_NE,
		NULL);

	/* create canvas for the circles & and load the circles */
	for(i=0;i<TEG_MAX_PLAYERS;i++)
	{
		players_color[i] = gnome_canvas_item_new(
			gnome_canvas_root(GNOME_CANVAS(mainstatus_canvas)),
			gnome_canvas_image_get_type (),
			"image", g_color_circles[TEG_MAX_PLAYERS],
			"x", 0.0,
			"y", 0.0,
			"width", (double) g_color_circles[TEG_MAX_PLAYERS]->rgb_width,
			"height", (double) g_color_circles[TEG_MAX_PLAYERS]->rgb_height,
			"anchor", GTK_ANCHOR_NW,
			NULL);
		gnome_canvas_item_hide( players_color[i] );
	}


	mainstatus_update();

	*window = mainstatus_canvas;
	gtk_widget_show( mainstatus_canvas );
	return TEG_STATUS_SUCCESS;

error:
	return TEG_STATUS_ERROR;
}

TEG_STATUS mainstatus_update_colors()
{
	int i;
	PLIST_ENTRY l;
	PJUGADOR pJ;

	if( ! mainstatus_canvas )
		return TEG_STATUS_ERROR;

	l = g_list_jugador.Flink;

	i=0;
	while( !IsListEmpty( &g_list_jugador ) && (l != &g_list_jugador) )
	{
		pJ = (PJUGADOR) l;

		if( pJ->color >= 0 && pJ->numjug >= 0 ) {
			gnome_canvas_item_show( players_color[i] );

			if( g_juego.whos_turn == pJ->numjug )
			{
				gnome_canvas_item_set(
					players_color[i],
					"image", g_color_circles[pJ->color],
					"x", (double) PLAYERS_COLORS_OFFSET - 2 + (i%3) * 14,
					"y", (double) 2 + 13 * (i<3?0:1),
					"width", (double) g_color_circles[pJ->color]->rgb_width + 4,
					"height", (double) g_color_circles[pJ->color]->rgb_height + 4,
					NULL);
			} else {
				gnome_canvas_item_set(
					players_color[i],
					"image", g_color_circles[pJ->color],
					"x", (double) PLAYERS_COLORS_OFFSET + (i%3) * 14,
					"y", (double) 4 + 13 * (i<3?0:1),
					"width", (double) g_color_circles[pJ->color]->rgb_width,
					"height", (double) g_color_circles[pJ->color]->rgb_height,
					NULL);
			}

			i++;
		}

		l = LIST_NEXT(l);

		if( i >= TEG_MAX_PLAYERS )
			break;
	}

	{
		PJUGADOR pJ;
		gnome_canvas_item_hide( color_started_item );
		if( g_juego.who_started_round >= 0 && g_juego.who_started_round < TEG_MAX_PLAYERS ) {

			if( jugador_whois( g_juego.who_started_round, &pJ ) == TEG_STATUS_SUCCESS )
			{
				gnome_canvas_item_set(
					color_started_item,
					"image", g_color_circles[pJ->color],
					NULL);
				gnome_canvas_item_show( color_started_item );
			}
		}
	}

	for( ; i < TEG_MAX_PLAYERS ; i++ )
			gnome_canvas_item_hide( players_color[i] );

	return TEG_STATUS_SUCCESS;
}

TEG_STATUS mainstatus_update()
{
	char buffer[256];
	static int offset_right=-1, offset_left=-1;
	static GnomeCanvasItem *gamestatus_item = NULL;
	static GnomeCanvasItem *country_item = NULL;
	static GnomeCanvasItem *cont_item = NULL;


	if( ! mainstatus_canvas )
		return TEG_STATUS_ERROR;

	if( offset_left == -1 && offset_right == -1)
	{
		offset_left = gui_theme.toolbar_offset_left != -1 ? gui_theme.toolbar_offset_left : OFFSET_LEFT;
		offset_right = gui_theme.toolbar_offset_right != -1 ? gui_theme.toolbar_offset_right : OFFSET_RIGHT;
	}

	/* game status */
	if( gamestatus_item  )
		gtk_object_destroy( GTK_OBJECT(gamestatus_item) );

	gamestatus_item = gnome_canvas_item_new(
		gnome_canvas_root(GNOME_CANVAS(mainstatus_canvas)),
		gnome_canvas_text_get_type(),
		"text",_(g_estados[g_juego.estado]),
		"x", (double) (MAINSTATUS_X + offset_right),
		"y", (double) (MAINSTATUS_Y/2),
		"x_offset", (double) -1,
		"y_offset", (double) -1,
		"font", HELVETICA_14_BFONT,
		"fill_color", gui_theme.toolbar_custom && gui_theme.toolbar_text_color ? gui_theme.toolbar_text_color : "black",
		"anchor",GTK_ANCHOR_EAST,
		NULL);

	/* country */
	if( country_item  )
		gtk_object_destroy( GTK_OBJECT(country_item) );

	if( gui_private.country_i_am < 0 || gui_private.country_i_am >= PAISES_CANT )
		snprintf(buffer,sizeof(buffer)-1," ");
	else
		snprintf(buffer,sizeof(buffer)-1,"%s", paises_get_name(gui_private.country_i_am) );
		
	country_item = gnome_canvas_item_new(
		gnome_canvas_root(GNOME_CANVAS(mainstatus_canvas)),
		gnome_canvas_text_get_type(),
		"text",buffer,
		"x", (double) offset_left,
		"y", (double) (2),
		"x_offset", (double) -1,
		"y_offset", (double) -1,
		"font", HELVETICA_12_FONT,
		"fill_color", gui_theme.toolbar_custom && gui_theme.toolbar_text_color ? gui_theme.toolbar_text_color : "black",
		"anchor",GTK_ANCHOR_NW,
		NULL);

	/* continent */
	if( cont_item  )
		gtk_object_destroy( GTK_OBJECT(cont_item) );

	if( gui_private.country_i_am < 0 || gui_private.country_i_am >= PAISES_CANT )
		snprintf(buffer,sizeof(buffer)-1," ");
	else
		snprintf(buffer,sizeof(buffer)-1,"%s", cont_get_name( g_paises[gui_private.country_i_am].continente )  );
		
	cont_item = gnome_canvas_item_new(
		gnome_canvas_root(GNOME_CANVAS(mainstatus_canvas)),
		gnome_canvas_text_get_type(),
		"text",buffer,
		"x", (double) offset_left,
		"y", (double) (16),
		"x_offset", (double) -1,
		"y_offset", (double) -1,
		"font", HELVETICA_10_FONT,
		"fill_color", gui_theme.toolbar_custom && gui_theme.toolbar_text_color ? gui_theme.toolbar_text_color : "black",
		"anchor",GTK_ANCHOR_NW,
		NULL);

	/* round number */
	{
		char buffer[256];

		gnome_canvas_item_hide( round_number_item );
		if( g_juego.round_number >= 0 ) {

			sprintf(buffer,"%d",g_juego.round_number );

			gnome_canvas_item_set(
				round_number_item,
				"text",buffer,
				NULL);
			gnome_canvas_item_show( round_number_item );
		}
	}

	mainstatus_update_colors();

	return TEG_STATUS_SUCCESS;
}
